import pytest
from prefect.testing.standard_test_suites import BlockStandardTestSuite

from prefect_firebolt import FireboltCredentials
from prefect_firebolt.database import FireboltDatabase


@pytest.mark.parametrize("block", [FireboltCredentials, FireboltDatabase])
class TestAllBlocksAdhereToStandards(BlockStandardTestSuite):
    @pytest.fixture
    def block(self, block):
        return block
